import numpy as np
import matplotlib.pyplot as plt
from scipy import optimize

plt.rcParams['font.size'] = 40

R=8.314
Tc= 374.15 +273.15
Pc= 22.12*10**6
a=27*R**2*Tc**2/(64*Pc)
b=R*Tc/(8*Pc)
Vc=3*b   #Volume molaire critique
eps= 10**(-8)

def Vol(a,b,P,R,T):   #Donne une liste ordonnée des trois volumes intersection de l'isotherme de VdW à T 
                      #avec le pallier de pression à P
    Vc=3*b 
    A=a*P/(R*T)**2
    B=b*P/(R*T)
    p=A-((B+1)**2)/3
    q=-2*((B+1)**3)/27 +A*(B+1)/3-A*B
    D=-(4*p**3+27*q**2)
    j=np.exp(2*np.pi*1j/3)
    sol=[]
    for k in range(3):
        u=(j**k)*(0.5*(-q+(-D/27)**(1/2)))**(1/3)
        v=(j**(-k))*(0.5*(-q-(-D/27)**(1/2)))**(1/3)
        sol.append(((np.real(u+v+(B+1)/3))*R*T/(3*b*P))*Vc)
    sol.sort()
    return sol

def carre(a,b,P,R,T):
    v=Vol(a,b,P,R,T)
    return P*(v[2]-v[0])
    
def area(a,b,P,R,T):
    v=Vol(a,b,P,R,T)
    return R*T*np.log((v[2]-b)/(v[0]-b))+a*(1/v[2]-1/v[0])
    
def dichotomie (f , a , b , epsilon ):
 #""" donn ées : f continue sur un intervalle ,
 #a et b tels que f(a)f(b) <= 0
 #ré sultat : une approximation à epsilon près d'une solution de
 #l'é quation f(x)=0 obtenue par dichotomie """
    g , d = min (a , b ) , max (a , b )
    while d - g > epsilon :
        m = ( g + d )/2
        if f ( m ) == 0:
            return m
        elif f ( g ) * f ( m ) < 0:
            d = m
        else :
            g = m
    return ( g + d )/2

lst_T= [0.85,0.9,0.98,1,1.2,1.5]

#lst_T=[0.9]
lst_v = np.linspace(1.1*b, 8*Vc,500)/Vc  

for i in range(len(lst_T)):
    T=lst_T[i]
    lst_P=[]
    lst_dP=[]
    index=[]
    for v in lst_v:
        lst_P.append((R*T*Tc/(v*Vc-b)-a/(v*Vc)**2)/Pc)
        lst_dP.append(6/v**3-8*T/(3*(v-1/3)**2))
    for i in range(len(lst_dP)):
        if lst_dP[i]>0:
            index.append(i)
    zone_P=[]
    zone_V=[]
    for i in index:
        zone_P.append(lst_P[i])
        zone_V.append(lst_v[i])
    plt.xlim(1/3,8)
    plt.xlabel(r"Volume reduit $V_r$",fontsize=40)
    plt.ylabel(r"Pression reduite $P_r$"+"\n",fontsize=40)
    plt.ylim(0,2)
    vt=round(T*Tc)
    if T==1 :
        plt.plot(lst_v,lst_P,label = r"$T = T_C = $%s"%vt+" K",linewidth=4)
    else:
        plt.plot(lst_v,lst_P,label = r"$T = $%s"%vt+" K",linewidth=4)
    #plt.plot(zone_V,zone_P,color='black',linewidth=6)
plt.title('Transition liquide-vapeur pour un fluide de Van der Waals'+'\n')
plt.scatter([1],[1],linewidth=5)
plt.text(0.8,1.05,"Point critique")
# plt.annotate(r'$\chi_T<0$',xy=(1.12, 0.46),  xycoords='data',
#             xytext=(2, 0.25), textcoords='data',
#             arrowprops=dict(facecolor='black', shrink=0.05),
#             horizontalalignment='right', verticalalignment='top', fontsize=50
#             )
plt.legend()
plt.grid('on')
plt.show()




    # if len(index)!=0:
    #     T_bis=T*Tc
    #     P0=R*T_b
    #     P1=R*T_bis/(lst_v[index[len(index)-5]]*Vc-b)-a/(lst_v[index[len(index)-5]]*Vc)**2
    #     print(P0)
    #     print(Vol(a,b,P0,R,T_bis))
    #     print(P1)
    #     print(Vol(a,b,P1,R,T_bis))

   ##       fun = lambda P : carre(a,b,P,R,T_bis)-area(a,b,P,R,T_bis)
    #     sol=dichotomie(fun,P0,P1,eps)
    #     V=Vol(a,b,sol,R,T_bis)
    #     print(V)
    #     Y=[sol for elem in zone_V]
    #     plt.plot(zone_V,Y)
